@@ -20,7 +20,7 @@ test_that("can retrieve without dataset", {
   expect_equal(df, tibble(count = 32L))
 })
 
-test_that("can retrieve query in pieces", {
+test_that("can retrieve query in pieces and that quiet is respected", {
   con <- DBI::dbConnect(
     bigquery(),
     project = bq_test_project(),
@@ -30,16 +30,35 @@ test_that("can retrieve query in pieces", {
   res <- DBI::dbSendQuery(con, "SELECT cyl, mpg FROM mtcars")
   expect_equal(DBI::dbGetRowCount(res), 0L)
 
-  df <- DBI::dbFetch(res, 10)
+  expect_snapshot(DBI::dbFetch(res, NA), error = TRUE)
+
+  res@quiet <- FALSE
+  expect_snapshot(df <- DBI::dbFetch(res, 10))
+
   expect_equal(nrow(df), 10)
   expect_false(DBI::dbHasCompleted(res))
   expect_equal(DBI::dbGetRowCount(res), 10L)
 
-  df <- DBI::dbFetch(res, -1)
+  res@quiet <- TRUE
+  expect_snapshot(df <- DBI::dbFetch(res, -1))
   expect_equal(nrow(df), 22)
   expect_true(DBI::dbHasCompleted(res))
 })
 
+test_that("dbHasCompleted() is accurate if you fetch past end", {
+  con <- DBI::dbConnect(
+    bigquery(),
+    project = bq_test_project(),
+    dataset = "basedata"
+  )
+
+  res <- DBI::dbSendQuery(con, "SELECT cyl FROM mtcars LIMIT 5")
+  df <- DBI::dbFetch(res, 10)
+
+  expect_equal(DBI::dbGetRowCount(res), 5)
+  expect_true(DBI::dbHasCompleted(res))
+})
+
 test_that("can get metadata", {
   con <- DBI::dbConnect(
     bigquery(),
